# Contributing to MapD's Charting library
👍🎉 Thanks for taking the time to contribute! 🎉👍

Our team welcomes and appreciates your contributions to this codebase. Every contribution helps, and our team will make sure you're given proper credit for your efforts.

### Contributor License Agreement

In order to clarify the intellectual property license granted with Contributions from any person or entity, MapD must have a Contributor License Agreement (“CLA”) on file that has been signed by each Contributor, indicating agreement to the [Contributor License Agreement](CONTRIBUTOR_LICENSE_AGREEMENT.md). If you have not already done so, please complete and sign, then scan and email a pdf file of this Agreement to contributors@mapd.com. Please read the agreement carefully before signing and keep a copy for your records.

You can contribute in many ways:

### Types of Issues
- [🐞 Bugs](#reporting-bugs)
- [📖 Documentation](#improving-documentation)
- [🆕 Enhancements](#suggesting-enhancements)


# 🐞 Reporting Bugs

This section guides you through submitting a bug report. Following these guidelines helps maintainers and the community understand your report :pencil:, reproduce the behavior :computer:, and find related reports. :mag_right:

Before creating bug reports, look through [existing issues](https://github.com/mapd/mapd-charting/issues?q=is%3Aopen+is%3Aissue+label%3Abug) as you might find out that you don't need to create one and can just 👍 an existing issue. When you are creating a bug report, [include as many details as possible](#how-do-i-submit-a-good-bug-report). Fill out [the required template](ISSUE_TEMPLATE.md), the information it asks for helps us resolve issues faster.

### How Do I Submit A (Good) Bug Report?

Bugs are tracked as [GitHub issues](https://guides.github.com/features/issues/). Create an issue on that repository and provide the following information by filling in [the template](ISSUE_TEMPLATE.md).

Explain the problem and include additional details to help maintainers reproduce the problem:

* **Use a clear and descriptive title** for the issue to identify the problem.
* **Describe the exact steps which reproduce the problem** in as much detail as possible. When listing steps, don't just say *what* you did, but explain *how* you did it.
* **Provide specific examples to demonstrate the steps**. Include links to files or GitHub projects, or copy/pasteable snippets, which you use in those examples. If you're providing snippets in the issue, use [Markdown code blocks](https://help.github.com/articles/markdown-basics/#multiple-lines).
* **Describe the behavior you observed after following the steps** and point out what exactly is the problem with that behavior.
* **Explain which behavior you expected to see instead and why.**
* **Include screenshots and animated GIFs** which show you following the described steps and clearly demonstrate the problem. You can use [this tool](http://www.cockos.com/licecap/) to record GIFs on macOS and Windows, and [this tool](https://github.com/colinkeenan/silentcast) or [this tool](https://github.com/GNOME/byzanz) on Linux. However **gifs alone are insufficient** to reproduce bugs!


# 📖 Improving Documentation
If you notice anything incorrect or missing from our documentation, correct it and open a PR!

Correcting typos and clarifying key functions and APIs are two great ways to make this library easier for everyone to use.


# 🆕 Suggesting Enhancements

This section guides you through submitting an enhancement suggestion, ranging from minor improvements for existing functionality to completely new features. Following these guidelines helps maintainers and the community understand your suggestion :pencil: and find related suggestions. :mag_right:

**Perform a [cursory search](https://github.com/mapd/mapd-charting/issues?q=is%3Aopen+is%3Aissue+label%3Aenhancement)** to see if the enhancement has already been suggested. If it has, add a comment to the existing issue instead of opening a new one.

When you are creating an enhancement suggestion, [include as many details as possible](#how-do-i-submit-a-good-enhancement-suggestion). Fill in [the template](ISSUE_TEMPLATE.md), including the steps that you imagine you would take if the feature you're requesting existed.

### How Do I Submit A (Good) Enhancement Suggestion?

* **Use a clear and descriptive title** for the issue to identify the suggestion.
* **Provide a step-by-step description of the suggested enhancement** in as much detail as possible.
* **Provide specific examples to demonstrate the steps**. Include copy/pasteable snippets which you use in those examples, as [Markdown code blocks](https://help.github.com/articles/markdown-basics/#multiple-lines).
* **Describe the current behavior** and **explain which behavior you expected to see instead** and why.
* **Include screenshots and animated GIFs** which help you demonstrate the steps or point out which part the suggestion is related to. You can use [this tool](http://www.cockos.com/licecap/) to record GIFs on macOS and Windows, and [this tool](https://github.com/colinkeenan/silentcast) or [this tool](https://github.com/GNOME/byzanz) on Linux.
* **Explain why this enhancement would be useful** to most users and/or devs using this code.


# Your First Code Contribution

Unsure where to begin contributing? You can start by looking for issues tagged `beginner`:

* [Beginner issues](https://github.com/mapd/mapd-charting/issues?utf8=%E2%9C%93&q=is%3Aopen%20is%3Aissue%20label%3A%22beginner%22%20) - issues which should only require a few lines of code, and a test or two.

### Opening Pull Requests
0. Make sure to commit your built `dist/` files.
0. Give your PR a commit-worthy name, as we'll squash° the commits into that name.
0. Fill out the Pull Request checklist ☑️ that's automatically added to your PR, especially what issue(s) you're addressing.
0. GitHub will automatically check to make sure your code can be merged safely. If it can't, `git rebase master` and push your changes back up.
0. [TravisCI](travis-ci.com) 👷 will automatically check for lint, passing tests, and no decreases in code coverage. If anything fails, commit a fix and push up to rerun CI.
0. Once the branch can be merged and CI passes ✅, a core contributor will review the code and make any comments.
0. There will probably be a bit of back-and-forth during this process as your code changes in response to comments.
0. Once the PR is approved, we'll squash-merge it in! :trophy:

° Squashing makes reverting easier should that become necessary.

### Branch Naming

We use the following branch names to ensure the stability of our codebase.

#### `develop`

This is a long-living branch off of master where active development work gets merged in via feature/ or chore/ branches. Do not commit to this branch directly.

#### `master`

This is a long-living branch that only contains production-ready code. Stable work should be merged into this branch from a `release/` branch when that code is ready for production. Urgent work may be merged in via `hotfix/` branches.

#### `feature/`

Create a temporary `feature/your-cool-feature` branch off of `develop` whenever you want to submit work through the normal release cycle. Your branch lives for as long as it takes for the feature to be complete enough to merge into `develop`, at which point you should rebase `develop` one final time and open a pull request into `develop`.

#### `chore/`

Create a temporary `chore/your-maintenance-task` branch off of `develop` when you're factoring/rewriting production code or performing general maintenance to architecture, dependencies, or documentation. Use the same process for merging a `feature/`.

#### `release/`

Create a temporary `release/version` branch off of `develop` when there is a viable release candidate. This branch lives for as long as it takes for the release candidate to be ready for production. To prepare a release, bump the version number using `npm version major/minor/patch` and merge into `master`.

#### `hotfix/`

Create a temporary `hotfix/your-urgent-matter` branch when an urgent fix needs to be released without merging the code in `develop`. Merge this branch back into `master` when ready, follow the normal release process, then back-merge the hotfix into `develop`.

### Style Guide
We use an extensive linter to help prevent typos and to maintain a consistent style in the codebase. The linter runs whenever you run `npm test`. The [linter settings file contains justifications](../.eslintrc.json) for most rules, but we're open to suggestions if you're willing to make the change!


# Becoming a Maintainer
We may ask community members who've proven themselves with consistently excellent PR and issue contributions to join our **Maintainers** team. There they'll help curate issues, review pull requests, and keep improving the community they're leading.

As we grow our internal engineering team, we may consider hiring contributors, particularly those who've earned this level of trust.

👍🎉 Again, thanks for contributing! 🎉👍
